"""This module is used to gather all python wrappers used to describe resources in Novu API.

In this SDK, we choose to split the Novu API by business resource to simplify its complexity.
"""

from novu.api.blueprint import BlueprintApi
from novu.api.change import ChangeApi
from novu.api.environment import EnvironmentApi
from novu.api.event import EventApi
from novu.api.execution_detail import ExecutionDetailApi
from novu.api.feed import FeedApi
from novu.api.inbound_parse import InboundParseApi
from novu.api.integration import IntegrationApi
from novu.api.layout import LayoutApi
from novu.api.message import MessageApi
from novu.api.notification import NotificationApi
from novu.api.notification_group import NotificationGroupApi
from novu.api.notification_template import NotificationTemplateApi
from novu.api.organization import OrganizationApi
from novu.api.subscriber import SubscriberApi
from novu.api.tenant import TenantApi
from novu.api.topic import TopicApi

__all__ = [
    "BlueprintApi",
    "ChangeApi",
    "EnvironmentApi",
    "EventApi",
    "ExecutionDetailApi",
    "FeedApi",
    "InboundParseApi",
    "IntegrationApi",
    "LayoutApi",
    "MessageApi",
    "NotificationApi",
    "NotificationGroupApi",
    "NotificationTemplateApi",
    "OrganizationApi",
    "SubscriberApi",
    "TenantApi",
    "TopicApi",
]
