"""This module is used to gather all constants and magic numbers used in the Novu client."""

# API Endpoints
CHANGES_ENDPOINT = "/v1/changes"
ENVIRONMENTS_ENDPOINT = "/v1/environments"
EVENTS_ENDPOINT = "/v1/events/trigger"
EXECUTION_DETAILS_ENDPOINT = "/v1/execution-details"
FEEDS_ENDPOINT = "/v1/feeds"
INBOUND_PARSE_ENDPOINT = "/v1/inbound-parse"
INTEGRATIONS_ENDPOINT = "/v1/integrations"
LAYOUTS_ENDPOINT = "/v1/layouts"
MESSAGES_ENDPOINT = "/v1/messages"
NOTIFICATION_ENDPOINT = "/v1/notifications"
NOTIFICATION_GROUPS_ENDPOINT = "/v1/notification-groups"
NOTIFICATION_TEMPLATES_ENDPOINT = "/v1/notification-templates"
ORGANIZATION_ENDPOINT = "/v1/organizations"
SUBSCRIBERS_ENDPOINT = "/v1/subscribers"
TENANTS_ENDPOINT = "/v1/tenants"
TOPICS_ENDPOINT = "/v1/topics"
BLUEPRINTS_ENDPOINT = "/v1/blueprints"
WORKFLOW_ENDPOINT = "/v1/workflows"

# Datetime Format
DATETIME_FORMAT = "%Y-%m-%dT%H:%M:%S.%f%z"
